// Copyright 2008-2016 David Robillard <d@drobilla.net>
// SPDX-License-Identifier: ISC

#ifndef LV2_INSTANCE_ACCESS_H
#define LV2_INSTANCE_ACCESS_H

/**
   @defgroup instance-access Instance Access
   @ingroup lv2

   Access to the LV2_Handle of a plugin for UIs.

   See <http://lv2plug.in/ns/ext/instance-access> for details.

   @{
*/

// clang-format off

#define LV2_INSTANCE_ACCESS_URI "http://lv2plug.in/ns/ext/instance-access"  ///< http://lv2plug.in/ns/ext/instance-access

// clang-format on

/**
   @}
*/

#endif /* LV2_INSTANCE_ACCESS_H */
